<?php

/**
 * Display a list of characters for the currently logged in user.
 */
function char_controller_list_get(): void
{
	auth_only();
	must_have_character();

	$chars = char_list(user('id'));

	echo render('layouts/basic', ['view' => 'pages/chars/list', 'chars' => $chars, 'activeTab' => nav_tabs['chars']]);
}

/**
 * Handle an action from the character list page.
 */
function char_controller_list_post(): void
{
	auth_only();
	must_have_character();
	csrf_ensure();

	$char_id = (int) ($_POST['char_id'] ?? 0);
	$action = $_POST['action'] ?? '';

	// If the character ID is not a number, or the action is not a string, return a 400.
	if (!is_numeric($char_id) || !is_string($action)) router_error(400);

	// If the action is not one of the allowed actions, return a 400.
	if (!in_array($action, ['select', 'delete'])) router_error(400);

	// If the action is to select a character, change the user's selected character.
	if ($action === 'select') {
		// If the character ID is the current character, do nothing.
		if ($char_id === $_SESSION['user']['char_id'] || $char_id === 0) {
			flash('info', 'You are already using that character.');
			redirect('/characters');
		}

		// Make sure the character ID is valid.
		if (char_exists($char_id) === false) throw new Exception('Invalid character ID. (acccp)');

		// Make sure the user owns the character.
		if (!char_belongs_to_user($char_id, $_SESSION['user']['id'])) router_error(999);

		change_user_character($char_id);

		flash('success', 'Switched to character ' . char('name') . '!');
	}

	// If the action is to delete a character, move to the confirmation page.
	if ($action === 'delete') {
		// Make sure the character ID is valid.
		if (char_exists($char_id) === false) throw new Exception('Invalid character ID. (accdp)');

		// Make sure the user owns the character.
		if (!char_belongs_to_user($char_id, $_SESSION['user']['id'])) router_error(999);

		$char = char_find($char_id);

		echo render('layouts/basic', ['view' => 'pages/chars/delete', 'char' => $char, 'activeTab' => nav_tabs['chars']]);
		return;
	}

	redirect('/characters');
}

/**
 * Delete a character for the currently logged in user.
 */
function char_controller_delete_post(): void
{
	auth_only();
	must_have_character();
	csrf_ensure();

	$char_id = (int) ($_POST['char_id'] ?? 0);
	$name = $_POST['name'] ?? '';

	// If the character ID is not a number, return a 400.
	if (!is_numeric($char_id)) router_error(400);

	// Make sure the character ID is valid.
	if (char_exists($char_id) === false) throw new Exception('Invalid character ID. (acddp)');

	// Make sure the user owns the character.
	if (!char_belongs_to_user($char_id, $_SESSION['user']['id'])) router_error(999);

	$char = char_find($char_id);

	// Confirm the name matches the name of the character.
	if ($char['name'] !== $_POST['name']) {
		flash('error', 'Failed to delete character. Name confirmation did not match.');
		redirect('/characters');
	}

	// Delete the character
	char_delete($char_id);

	// If the character being deleted is the currently selected character, select the first character.
	if ($_SESSION['user']['char_id'] === $char_id) {
		$chars = char_list(user('id'));
		if (count($chars) > 0) change_user_character($chars[0]['id']);
	}

	flash('error', 'Character ' . $char['name'] . ' deleted.');
	redirect('/characters');
}

/**
 * Form to create your first character.
 */
function char_controller_create_first_get(): void
{
	auth_only();

	// If the user already has a character, redirect them to the main page.
	if (char_count(user('id')) > 0) redirect('/');

	echo render('layouts/basic', ['view' => 'pages/chars/first', 'activeTab' => nav_tabs['chars']]);
}

/**
 * Create a character for the currently logged in user.
 */
function char_controller_create_post(): void
{
	auth_only();
	csrf_ensure();

	$errors = [];

	$name = $_POST['name'] ?? '';

	// Trim the input.
	$name = trim($name);

	/*
		A name is required.
		A name must be between 3 and 18 characters.
		A name must contain only alphanumeric characters and spaces.
	*/
	if (empty($name) || strlen($name) < 3 || strlen($name) > 18 || !ctype_alnum(str_replace(' ', '', $name))) {
		$errors['name'][] = 'Name is required and must be between 3 and 18 characters long and contain only alphanumeric characters and spaces.';
	}

	/*
		A character's name must be unique.
	*/
	if (char_name_exists($name)) $errors['name'][] = 'Name is already taken.';

	// If there are errors at this point, send them to the page with errors flashed.
	if (!empty($errors)) {
		flash('alert-cl2', ['errors', $errors]);
		redirect('/characters');
	}

	// Create the character
	$char = char_create(user('id'), $name);
	if ($char === false) router_error(400);

	// Create the auxiliary tables
	char_location_create($char);
	char_gear_create($char);

	// Set the character as the user's selected character
	change_user_character($char);

	flash('alert_character_list_1', ['success', 'Character ' . $name . ' created!']);
	redirect('/characters');
}

